# Make a package.

def registerTestSetup():
    from schooltool.testing import registry

    def addInterventionRoot(app):
        from schooltool.intervention import intervention
        app['schooltool.interventions'] = intervention.InterventionRoot()
    registry.register('ApplicationContainers', addInterventionRoot)

registerTestSetup()
del registerTestSetup

from zope.i18nmessageid import MessageFactory
InterventionGettext = MessageFactory("schooltool.intervention")

import schooltool.common

schooltool.common.register_lauchpad_project(__package__, 'schooltool.intervention')

