/*
 * Stylesheet for SLA intervention section view
 */

div.tabmenu {
  display: block;
  float: left;
  margin: 0;
  font-weight: bold;
  font-size: 1em;
}

div.tabmenu p {
  display: inline;
  float: left;
  text-decoration: none;
  padding: 0.3em 0.5em;
  margin: 0px;
  border: 1px solid black;
}

div.tabmenu a {
  background: #eae8e3;
  display: inline;
  float: left;
  padding: 0.2em 0.5em;
  margin: 0.2em 0px 0px 0px;
  border: 1px solid black;
}

table.students_nav {
  border-spacing: 0;
}

table.students_nav td {
  padding: 0 0.2em 0.5em 0;
}

table.students_nav th {
  padding: 0;
  border-bottom: 1px solid black;
}

div.student_nav_link {
  display: inline;
}

div.student_nav_link a {
  font-weight: bold;
  font-size: 1em;
  display: inline;
  float: left;
  padding: 0.2em 0.5em;
  margin: 0.2em 0px 0px 0px;
}
